import { TextInput, DialogService, LoadingService } from "@zeedhi/common";
import { Metadata, Http } from '@zeedhi/core';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';

export class Utilities {

    public static getNameDynamic(name: string) {
        const date = new Date();
        return name + date.getTime();
    }

    public static lightColorHexadecimalValidity(color: String) {
        const hex = color.replace('#', '');

        const r = parseInt(hex.substr(0, 2), 2);
        const g = parseInt(hex.substr(2, 2), 2);
        const b = parseInt(hex.substr(4, 2), 2);

        const lum = (r * 299 + g * 587 + b * 114) / 1000;
    }

    public static roundFloat(num: number, decimalPlaces: number = 2) {
        const tenPower = 10 ** decimalPlaces;
        return Math.round((num + Number.EPSILON) * tenPower) / tenPower;
    }

    public static fixBugHintOnfocus(component: TextInput, element: HTMLElement | undefined, count = 1) {
        setTimeout(() => {
            if (element) {
                const messages = element.querySelector('.v-messages__message');
                if (messages) {
                    if (component.hint) {
                        messages.innerHTML = component.hint;
                    }

                } else if (count < 5) {
                    count += 1;
                    Utilities.fixBugHintOnfocus(component, element, count);
                }
            }

        });
    }

    public static async genericBeforeSaveTekGrid(params: any, route: string, tekGridName: string) {
        params.event.preventDefault();
        try {
            LoadingService.show();
            await Http.post(route, {
                "requestType": "DataSet",
                "dataset": params.component.getEditedRows()
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            params.component.cancelEditedRows();
            Metadata.getInstance<TekGrid>(tekGridName).datasource.get();
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(e);
        }
    }

    // Before Delete Methods

    public static deleteRoute: string;
    public static deleteTekGridName: string;
    public static deleteRequestType: string;

    public static genericBeforeDeleteTekGrid(params: any, route: string, tekGridName: string, requestType: string = 'DataSet') {
        params.event.preventDefault();
        this.deleteRoute = route;
        this.deleteTekGridName = tekGridName;
        this.deleteRequestType = requestType;
        DialogService.show({
          name: 'delete-dialog_' + tekGridName,
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', outline: true, events: { click: () => DialogService.hide(), }, },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: Utilities.deleteTekGrid, },
          ],
        });
    }

    private static deleteTekGrid: any = {
        click: () => Utilities.doDeleteTekGrid(),
    };

    private static async doDeleteTekGrid() {
        DialogService.hide();
        try {
            const grid = Metadata.getInstance<TekGrid>(this.deleteTekGridName);
            const requestTypeAttr = this.deleteRequestType === "DataSet" ? "dataset" : "row";
            LoadingService.show();
            await Http.post(Utilities.deleteRoute, {
                "requestType": this.deleteRequestType,
                [requestTypeAttr]: grid.selectedRows
            });
            LoadingService.hide();
            CustomAlertService.deleteSuccessAlert();
            grid.datasource.get();
            grid.selectedRows = [];
        } catch (e) {
            LoadingService.hide();
            CustomAlertService.errorAlert(e);
        }
        this.deleteRoute = '';
        this.deleteTekGridName = '';
    }

}
